package org.springblade.modules.api.controller;

import com.alibaba.fastjson.JSONObject;
import com.alipay.api.AlipayApiException;
import com.alipay.api.AlipayClient;
import com.alipay.api.AlipayResponse;
import com.alipay.api.request.AlipayFundAuthOperationDetailQueryRequest;
import com.alipay.api.request.AlipayFundAuthOrderAppFreezeRequest;
import com.alipay.api.request.AlipayTradeAppPayRequest;
import com.alipay.api.request.AlipayTradePayRequest;
import com.alipay.api.response.AlipayFundAuthOperationDetailQueryResponse;
import com.alipay.api.response.AlipayFundAuthOrderAppFreezeResponse;
import com.alipay.api.response.AlipayTradePayResponse;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.AllArgsConstructor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springblade.common.constant.CommonConstant;
import org.springblade.common.utils.NotifyUtils;
import org.springblade.core.log.annotation.ApiLog;
import org.springblade.core.log.logger.BladeLogger;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.StringUtil;
import org.springblade.modules.finance.entity.Order;
import org.springblade.modules.finance.pay.entity.AlipayProperties;
import org.springblade.modules.finance.service.IOrderService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;

@RestController
@AllArgsConstructor
@RequestMapping(CommonConstant.API_URL + "/aliPay/sdk")
@Api(value = "支付宝SDK", tags = "95.支付宝SDK")
public class AliPay01Controller {

	private static final Logger log = LoggerFactory.getLogger(AliPay01Controller.class);

	//异步回调路径 https://wine.gzzzyd.com/api/v1/aliPay/sdk/notify_url
	private static final String NOTIFY_URL = "https://wine.gzzzyd.com/api/v1/aliPay/sdk/notify_url";

	@Autowired
	private AlipayClient client;//支付宝请求sdk客户端

	@Autowired
	private AlipayProperties properties;//支付宝sdk配置

	private final IOrderService orderService;

	@Autowired
	private BladeLogger logger;

	@ApiLog("发起支付-直接交易")
	@PostMapping("/pay_001")
	@ApiOperation(value = "统一收单线下交易查询", notes = "当面付")
	@ApiOperationSupport(order = 1)
	public R pay001(@ApiParam(value = "订单号", required = true) @RequestParam String outTradeNo,
					@ApiParam(value = "支付渠道:aliPay支付宝，wechatPay微信支付，unionPay银联 ", required = true) @RequestParam String payFor) throws AlipayApiException {
		Order order = orderService.getOne(Wrappers.<Order>lambdaQuery().eq(Order::getOutTradeNo, outTradeNo));
		if (order == null){
			return R.fail("请求参数错误！");
		}
		if(StringUtil.isEmpty(payFor)){
			return R.fail("请选择支付方式！");
		}
		if("aliPay".equals(payFor)){
			JSONObject data = new JSONObject();
			data.put("out_trade_no", outTradeNo); //商户订单号
			//产品码, APP支付 QUICK_MSECURITY_PAY, PC支付 FAST_INSTANT_TRADE_PAY, 移动H5支付 QUICK_WAP_PAY
			data.put("product_code", "QUICK_MSECURITY_PAY");
			data.put("total_amount", "0.01"); //订单金额
			data.put("subject", "中宏酒业"); //订单标题
			//APP支付
			AlipayTradeAppPayRequest request = new AlipayTradeAppPayRequest();
			request.setNotifyUrl(NOTIFY_URL); //异步通知地址
			request.setBizContent(data.toJSONString()); //业务参数
			//APP支付
			try {
				AlipayResponse response = client.sdkExecute(request);
				return R.data(response.getBody());
			} catch (AlipayApiException e) {
				e.printStackTrace();
			}
			return R.fail("发起支付失败！");
		}
		return R.fail("该支付方式暂不支持！");
	}

	@ApiLog("发起支付-预授权")
	@PostMapping("/pay_002")
	@ApiOperation(value = "统一收单交易支付接口", notes = "预授权")
	@ApiOperationSupport(order = 2)
	public R pay002(@ApiParam(value = "订单号", required = true) @RequestParam String outTradeNo,
					@ApiParam(value = "授权订单号", required = true) @RequestParam String authCode,
					@ApiParam(value = "支付渠道:aliPay支付宝，wechatPay微信支付，unionPay银联 ", required = true) @RequestParam String payFor) throws AlipayApiException {
		Order order = orderService.getOne(Wrappers.<Order>lambdaQuery().eq(Order::getOutTradeNo, outTradeNo));
		if (order == null){
			return R.fail("请求参数错误！");
		}
		if(StringUtil.isEmpty(payFor)){
			return R.fail("请选择支付方式！");
		}
		if("aliPay".equals(payFor)){
			JSONObject data = new JSONObject();
			data.put("out_trade_no", outTradeNo); //商户订单号
			//产品码, APP支付 QUICK_MSECURITY_PAY, PC支付 FAST_INSTANT_TRADE_PAY, 移动H5支付 QUICK_WAP_PAY
			data.put("product_code", "QUICK_MSECURITY_PAY");
			data.put("total_amount", "0.01"); //订单金额
			data.put("subject", "中宏酒业"); //订单标题
			data.put("scene", "bar_code");
			data.put("auth_code", authCode);
			//APP支付
			AlipayTradePayRequest request = new AlipayTradePayRequest();
			request.setNotifyUrl(NOTIFY_URL); //异步通知地址
			request.setBizContent(data.toJSONString()); //业务参数
			//APP支付
			try {
				AlipayTradePayResponse response = client.sdkExecute(request);
				return R.data(response.getBody());
			} catch (AlipayApiException e) {
				e.printStackTrace();
			}
			return R.fail("发起支付失败！");
		}
		return R.fail("该支付方式暂不支持！");
	}

	@PostMapping("/pay_003")
	@ApiOperation(value = "线上资金授权冻结接口", notes = "支付请求")
	@ApiOperationSupport(order = 3)
	public R pay003(@ApiParam(value = "订单号", required = true) @RequestParam("outTradeNo") String outTradeNo) throws AlipayApiException {
		JSONObject data = new JSONObject();
		data.put("out_order_no", outTradeNo);
		data.put("out_request_no", outTradeNo);
		data.put("order_title","预授权冻结");
		data.put("amount",0.01);
		data.put("product_code","PRE_AUTH_ONLINE"); //销售产品码。支付宝预授权产品固定为 PRE_AUTH_ONLINE
		data.put("payee_user_id", "2088341967443472");
		data.put("timeout_express","30m");
		//APP支付
		AlipayFundAuthOrderAppFreezeRequest request = new AlipayFundAuthOrderAppFreezeRequest();
		request.setNotifyUrl(NOTIFY_URL); //异步通知地址
		request.setBizContent(data.toJSONString()); //业务参数
		//APP支付
		AlipayFundAuthOrderAppFreezeResponse response = client.sdkExecute(request);
		if(response.isSuccess()){
			return R.data(response.getBody());
		}
		return R.fail("授权失败！");
	}

	@GetMapping("/pay_004")
	@ApiOperation(value = "资金授权操作查询接口", notes = "支付请求")
	@ApiOperationSupport(order = 3)
	public R pay004(@ApiParam(value = "订单号", required = true) @RequestParam String outTradeNo,
					@ApiParam(value = "查询明细，1查询冻结明细，2查询解冻明细,3查询支付明细时", required = true) @RequestParam Integer type) throws AlipayApiException {
		JSONObject data = new JSONObject();
		if(type == 1){ //查询冻结明细
			data.put("out_order_no",outTradeNo);
			data.put("operation_type","FREEZE");
		}else if(type == 2){ //查询解冻明细
			data.put("out_order_no",outTradeNo);
			data.put("operation_type","UNFREEZE");
		}else if(type == 3){ //查询支付明细时
			data.put("out_order_no",outTradeNo);
			data.put("operation_type","PAY");
		}else{
			return R.fail("参数错误！");
		}
		// 设置整体请求参数
		//APP支付
		AlipayFundAuthOperationDetailQueryRequest request = new AlipayFundAuthOperationDetailQueryRequest();
		request.setBizContent(data.toString());
		//APP支付
		AlipayFundAuthOperationDetailQueryResponse response = client.sdkExecute(request);
		if(response.isSuccess()){
			return R.data(response.getBody());
		}
		return R.fail("授权失败！");
	}

	@RequestMapping(value = "/notify_url", method = {RequestMethod.GET, RequestMethod.POST})
	@ApiOperation(value = "异步通知相应", notes = "异步通知相应")
	@ApiOperationSupport(order = 100)
	public String notify(HttpServletRequest request) {
		logger.info("支付宝回调-预授权", request.getParameterMap().toString());
		if (!NotifyUtils.rsaCheck(properties, request.getParameterMap())) {
			//这里处理验签失败
		}
		log.info("支付宝回调-预授权");
		log.info(request.getParameterMap().toString());
		request.getParameter("trade_no");//获取请求参数中的商户订单号
		return "success";
	}
}
