/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.modules.finance.service.impl;

import cn.hutool.core.util.NumberUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import org.springblade.common.cache.ParamCache;
import org.springblade.common.constant.CommonConstant;
import org.springblade.core.log.exception.ServiceException;
import org.springblade.core.log.logger.BladeLogger;
import org.springblade.core.tool.utils.DateUtil;
import org.springblade.modules.api.request.*;
import org.springblade.modules.api.response.ConsignResponse;
import org.springblade.modules.finance.entity.Order;
import org.springblade.modules.finance.entity.OrderConsign;
import org.springblade.modules.finance.entity.OrderGoods;
import org.springblade.modules.finance.mapper.OrderConsignMapper;
import org.springblade.modules.finance.vo.OrderVO;
import org.springblade.modules.finance.mapper.OrderMapper;
import org.springblade.modules.finance.service.IOrderService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springblade.modules.shopping.entity.Consign;
import org.springblade.modules.shopping.entity.Goods;
import org.springblade.modules.shopping.entity.GoodsSpecs;
import org.springblade.modules.shopping.entity.SpecItems;
import org.springblade.modules.shopping.mapper.ConsignMapper;
import org.springblade.modules.shopping.mapper.GoodsMapper;
import org.springblade.modules.shopping.mapper.GoodsSpecsMapper;
import org.springblade.modules.shopping.mapper.SpecItemsMapper;
import org.springblade.modules.system.entity.User;
import org.springblade.modules.system.entity.UserApp;
import org.springblade.modules.system.mapper.UserMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.ijpay.core.kit.PayKit;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * 订单表 服务实现类
 *
 * @author BladeX
 * @since 2022-02-13
 */
@Service
public class OrderServiceImpl extends ServiceImpl<OrderMapper, Order> implements IOrderService {

	@Autowired
	private UserMapper userMapper;

	@Autowired
	private GoodsMapper goodsMapper;

	@Autowired
	private GoodsSpecsMapper goodsSpecsMapper;

	@Autowired
	private SpecItemsMapper specItemsMapper;

	@Autowired
	private ConsignMapper consignMapper;

	@Autowired
	private OrderConsignMapper orderConsignMapper;

	private BladeLogger logger;

	@Override
	public IPage<OrderVO> selectOrderPage(IPage<OrderVO> page, OrderVO order) {
		return page.setRecords(baseMapper.selectOrderPage(page, order));
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public synchronized String commonOrderInsert(OrderCommonSaveRequest request, String tenantId) {
		UserApp userApp = new UserApp();
		UserApp query = userApp.selectOne(Wrappers.<UserApp>lambdaQuery().eq(UserApp::getUserId,request.getUserId()));
		if (query == null){
			throw new ServiceException("请重新登录！");
		}
		if (query.getState() != 1){
			throw new ServiceException("登录账号已冻结！");
		}
		User user = userMapper.selectById(query.getUserId());
		//生成订单号
		String outTradeNo = PayKit.generateStr();
		Order order = new Order();
		order.setTenantId(tenantId);
		if(user != null){
			order.setUserName(user.getName());
		}
		order.setOutTradeNo(outTradeNo);
		order.setUserId(query.getUserId());
		order.setOrderType(1);//普通订单
		order.setPayFlag(1); //待付款
		order.setPayFee(request.getPayFee());
		order.setUserAddress(request.getUserAddress());
		order.setTelPhone(request.getTelPhone());
		order.setDistributeMode(request.getDistributeMode());
		order.setDeliveryStaff(request.getDeliveryStaff());
		order.setReceivingName(request.getReceivingName());
		order.setWlPrice(request.getWlPrice());
		Double totalFee = 0.0;
		Double totalTicket = 0.0;
		List<OrderGoodsSaveRequest> goodsList = request.getGoodsList();
		if (goodsList == null || goodsList.size() == 0){
			return "";
		}
		for (OrderGoodsSaveRequest item: goodsList) {
			Goods goods = goodsMapper.selectById(item.getGoodsId());
			if (goods == null){
				throw new ServiceException("商品不存在！");
			}
			if (goods.getIsSale() == 1){
				throw new ServiceException("该商品未上架！");
			}
			if (goods.getGoodsStock() == 0){
				throw new ServiceException("该商品库存不足！");
			}
			OrderGoods orderGoods = new OrderGoods();
			orderGoods.setGoodsImg(goods.getGoodsImg());
			Double goodsPrice = goods.getGoodsPrice();
			Double ticketNum = goods.getTicketNum();
			GoodsSpecs goodsSpecs = goodsSpecsMapper.selectById(item.getGoodsSpecIds());
			if (goodsSpecs != null){
				//获取商品规格数据
				orderGoods.setGoodsPrice(goodsSpecs.getSpecPrice());
				goodsPrice = goodsSpecs.getSpecPrice();
				ticketNum = goodsSpecs.getTicketNum();
				SpecItems specItems = specItemsMapper.selectById(goodsSpecs.getSpecIds());
				if (specItems != null){
					orderGoods.setGoodsSpecNames(specItems.getItemName());
					if(orderGoods.getGoodsImg() != null){
						orderGoods.setGoodsImg(specItems.getItemImg());
					}
				}
			}
			orderGoods.setOutTradeNo(outTradeNo);
			orderGoods.setGoodsImg(goods.getGoodsImg());
			orderGoods.setGoodsName(goods.getGoodsName());
			orderGoods.setGoodsNum(item.getGoodsNum());
			orderGoods.setGoodsSpecIds(item.getGoodsSpecIds());
			orderGoods.setTicketVal(goods.getTicketNum());
			Double specPrice = NumberUtil.toDouble(NumberUtil.mul(goodsPrice, item.getGoodsNum()));
			totalFee = NumberUtil.add(totalFee, specPrice);
			Double specTicket = NumberUtil.toDouble(NumberUtil.mul(ticketNum, item.getGoodsNum()));
			totalTicket = NumberUtil.add(totalTicket, specTicket);
			boolean flg = orderGoods.insert();
			if (!flg){
				throw new ServiceException("操作失败！");
			}
		};
		order.setTotalFee(totalFee);
		order.setTotalTicket(totalTicket);
		int i = baseMapper.insert(order);
		if (i != 1){
			throw new ServiceException("操作失败！");
		}
		return outTradeNo;

	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public synchronized String consignOrderInsert(OrderConsignSaveRequest request, String tenantId) {
		UserApp userApp = new UserApp();
		UserApp query = userApp.selectOne(Wrappers.<UserApp>lambdaQuery().eq(UserApp::getUserId,request.getUserId()));
		if (query == null){
			throw new ServiceException("请重新登录！");
		}
		if (query.getState() != 1){
			throw new ServiceException("登录账号已冻结！");
		}
		if (query.getCreditScore() < 100){
			throw new ServiceException("您的信誉分不够！");
		}
		//已抢次数
		int count = orderConsignMapper.selectCount(Wrappers.<OrderConsign>lambdaQuery().last(" where to_days(create_time)= to_days(now())"));
		//可抢次数
		float k = (float) (query.getCreditScore() / 100);
		if(count >= k){
			throw new ServiceException("您抢购的次数已用完！");
		}
		//获取寄售商品数据
		Consign consign = consignMapper.selectById(request.getConsignId());
		if (consign == null){
			throw new ServiceException("该寄售商品不存在！");
		}
		if(consign.getIsSale() == 1){
			throw new ServiceException("订单下架");
		}
		if(consign.getIsDelivery() == 2){
			throw new ServiceException("订单已交割");
		}
		if(consign.getConsignFor() == 1){
			//后台生成的抢购
			if(consign.getNowStock() == 0){
				throw new ServiceException("亲，好物已抢完欢迎再来！");
			}
		}else if(consign.getConsignFor() == 2){
			//用户的寄售订单已寄售
			if(consign.getConsignState() != 1){
				throw new ServiceException("亲，好物已抢完欢迎再来！");
			}
		}else{
			throw new ServiceException("系统错误，请联系管理员！");
		}
		//查询商品信息
		Goods goods = goodsMapper.selectById(request.getGoodsId());
		if (goods == null){
			throw new ServiceException("商品不存在！");
		}
		User user = userMapper.selectById(query.getUserId());
		//生成订单号
		String outTradeNo = PayKit.generateStr();
		Order order = new Order();
		order.setTenantId(tenantId);
		if(user != null){
			order.setUserName(user.getName());
		}
		order.setOutTradeNo(outTradeNo);
		order.setUserId(query.getUserId());
		order.setOrderType(3);//寄售订单
		order.setPayFlag(1); //待付款
		order.setPayFee(request.getPayFee());
		order.setUserAddress(request.getUserAddress());
		order.setTelPhone(request.getTelPhone());
		order.setDistributeMode(2);//默认物流配送
		order.setDeliveryStaff("物流配送");
		order.setReceivingName(request.getReceivingName());
		int i = baseMapper.insert(order);
		if (i == 1){
			OrderGoods orderGoods = new OrderGoods();
			orderGoods.setOutTradeNo(outTradeNo);
			orderGoods.setGoodsImg(goods.getGoodsImg());
			orderGoods.setGoodsName(goods.getGoodsName());
			orderGoods.setGoodsNum(1);
			orderGoods.setGoodsSpecIds(request.getGoodsSpecIds());
			boolean flg = orderGoods.insert();
			if (!flg){
				throw new ServiceException("操作失败！");
			}
			//添加寄售订单数据
			OrderConsign orderConsign = new OrderConsign();
			orderConsign.setTotalNum(1);//默认寄售数量为1
			//判断来源为寄售
			if(consign.getConsignFor() == 2){
				//更新寄售状态
				Consign updateConsign = new Consign();
				updateConsign.setId(consign.getId());
				updateConsign.setConsignState(2); //
				updateConsign.setConsignType(request.getConsignType());//修改寄售类型（寄售方式）
				updateConsign.setNowStock(0); //当前库存为0
				int c =  consignMapper.updateById(updateConsign);
				if (c != 1){
					throw new ServiceException("操作失败！");
				}
			}else if(consign.getConsignFor() == 1){ //来源为后台
				//减少库存数客户到达一定的数修改寄售状态为已寄售
				Consign updateConsign = new Consign();
				updateConsign.setId(consign.getId());
				if (consign.getConsignType() == 2){
					//寄售状态为已寄售
					updateConsign.setConsignState(2); //待付款
				}
				updateConsign.setConsignType(request.getConsignType());
				updateConsign.setNowStock(consign.getNowStock() - 1);
				int c =  consignMapper.updateById(updateConsign);
				if (c != 1){
					throw new ServiceException("操作失败！");
				}
			}
			orderConsign.setConsignId(consign.getId()); //寄售ID
			orderConsign.setOutTradeNo(outTradeNo);
			orderConsign.setGoodsId(consign.getGoodsId());
			orderConsign.setGoodsName(goods.getGoodsName());
			orderConsign.setConsignType(request.getConsignType());
			orderConsign.setPayUserId(request.getUserId()); //支付人
			if(request.getConsignType() == 1){
				orderConsign.setConsignDays(0);
			}else if(request.getConsignType() == 2){
				orderConsign.setConsignDays(request.getConsignDays());
			}else{
				throw new ServiceException("请求参数错误！");
			}
			orderConsign.setConsignUserId(consign.getConsignUserId());
			orderConsign.setTotalPrice(request.getPayFee());
			orderConsign.setConsignPrice(request.getPayFee());
			int j = orderConsignMapper.insert(orderConsign);
			if(j != 1){
				throw new ServiceException("操作失败！");
			}
			return outTradeNo;
		}
		return "";
	}

	@Override
	public boolean consignSelfTake(OrderTakeRequest request) {
		Consign consign = consignMapper.selectById(request.getConsignId());
		if(consign != null
			&& Objects.equals(consign.getConsignUserId(), request.getUserId())
			&& consign.getOrderId() != null){
			Order order = baseMapper.selectById(consign.getOrderId());
			if (order != null){
				Order param = new Order();
				param.setId(order.getId());
				param.setPayFlag(3); //待收货
				param.setSendTime(DateUtil.now());
				int i = baseMapper.updateById(order);
				return i == 1;
			}
		}
		return false;
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public ConsignResponse consignOrderInsert(OrderConsignRandomRequest request, String tenantId) {
		ConsignResponse response = new ConsignResponse();
		UserApp userApp = new UserApp();
		UserApp query = userApp.selectOne(Wrappers.<UserApp>lambdaQuery().eq(UserApp::getUserId,request.getUserId()));
		if (query == null){
			throw new ServiceException("请重新登录！");
		}
		if (query.getState() != 1){
			throw new ServiceException("登录账号已冻结！");
		}
		if (query.getCreditScore() < 100){
			throw new ServiceException("您的信誉分不够！");
		}
		//是特殊账号的不判断的抢购次数
		if (query.getIsAuth() != 2){
			//已抢次数 （可抢订单数/日）
			int count = orderConsignMapper.selectCount(Wrappers.<OrderConsign>lambdaQuery().last(" where to_days(create_time)= to_days(now())"));
			//可抢次数
			double k = (double)query.getCreditScore() / 100;
			Integer num = NumberUtil.roundDown(k, 2).intValue() - (count + 1);
			response.setNum(num);
			if(count >= k){
				throw new ServiceException("您抢购的次数已用完！");
			}
		}
		//获取寄售商品数据
		Consign consign = consignMapper.randomConsign(request.getConsignConfigId());
		if (consign == null){
			throw new ServiceException("亲，好物已抢完欢迎再来！");
		}
		//查询商品信息
		Goods goods = goodsMapper.selectById(consign.getGoodsId());
		if (goods == null){
			throw new ServiceException("商品不存在！");
		}
		User user = userMapper.selectById(query.getUserId());
		//生成订单号
		String outTradeNo = PayKit.generateStr();
		Order order = new Order();
		order.setTenantId(tenantId);
		if(user != null){
			order.setUserName(user.getName());
		}
		order.setOutTradeNo(outTradeNo);
		order.setUserId(query.getUserId());
		order.setOrderType(3);//寄售订单
		order.setPayFlag(1); //待付款
		order.setPayFee(consign.getConsignPrice());
		order.setUserAddress(request.getUserAddress());
		order.setTelPhone(request.getTelPhone());
		order.setDistributeMode(2);//默认物流配送
		order.setDeliveryStaff("物流配送");
		order.setReceivingName(request.getReceivingName());
		int i = baseMapper.insert(order);
		if (i == 1){
			OrderGoods orderGoods = new OrderGoods();
			orderGoods.setOutTradeNo(outTradeNo);
			orderGoods.setGoodsImg(goods.getGoodsImg());
			orderGoods.setGoodsName(goods.getGoodsName());
			orderGoods.setGoodsNum(1);
			boolean flg = orderGoods.insert();
			if (!flg){
				throw new ServiceException("操作失败！");
			}
			//添加寄售订单数据
			OrderConsign orderConsign = new OrderConsign();
			orderConsign.setTotalNum(1);//默认寄售数量为1
			//判断来源为寄售
			if(consign.getConsignFor() == 2){
				//更新寄售状态
				Consign updateConsign = new Consign();
				updateConsign.setId(consign.getId());
				updateConsign.setConsignState(2); //
				updateConsign.setConsignType(request.getConsignType());//修改寄售类型（寄售方式）
				updateConsign.setNowStock(0); //当前库存为0
				int c =  consignMapper.updateById(updateConsign);
				if (c != 1){
					throw new ServiceException("操作失败！");
				}
			}else if(consign.getConsignFor() == 1){ //来源为后台
				//减少库存数客户到达一定的数修改寄售状态为已寄售
				Consign updateConsign = new Consign();
				updateConsign.setId(consign.getId());
				if (consign.getConsignType() == 2){
					//寄售状态为已寄售
					updateConsign.setConsignState(2); //待付款
				}
				updateConsign.setConsignType(request.getConsignType());
				updateConsign.setNowStock(consign.getNowStock() - 1);
				int c =  consignMapper.updateById(updateConsign);
				if (c != 1){
					throw new ServiceException("操作失败！");
				}
			}
			orderConsign.setConsignId(consign.getId()); //寄售ID
			orderConsign.setOutTradeNo(outTradeNo);
			orderConsign.setGoodsId(consign.getGoodsId());
			orderConsign.setGoodsName(goods.getGoodsName());
			orderConsign.setConsignType(request.getConsignType());
			orderConsign.setPayUserId(request.getUserId()); //支付人
			if(request.getConsignType() == 1){
				orderConsign.setConsignDays(0);
			}else if(request.getConsignType() == 2){
				orderConsign.setConsignDays(consign.getConsignDays() + 1);
			}else{
				throw new ServiceException("请求参数错误！");
			}
			orderConsign.setConsignUserId(consign.getConsignUserId());
			orderConsign.setTotalPrice(consign.getConsignPrice());
			orderConsign.setConsignPrice(consign.getConsignPrice());
			int j = orderConsignMapper.insert(orderConsign);
			if(j != 1){
				throw new ServiceException("操作失败！");
			}
			response.setOutTradeNo(outTradeNo);
			return response;
		}
		return response;
	}

	@Override
	public boolean payCommonSuccess(String outTradeNo, String payCode) {
		Order queryOrder = baseMapper.selectOne(Wrappers.<Order>lambdaQuery().eq(Order::getOutTradeNo, outTradeNo));
		if (queryOrder == null){
			logger.error("订单异常", "无法获取订单数据！订单号：" + outTradeNo);
			return false;
		}
		if(queryOrder.getPayFlag() == 2){
			logger.info("订单异常", "订单已支付成功，不能重复发起支付！订单号：" + outTradeNo);
			return false;
		}
		OrderConsign query = orderConsignMapper.selectOne(Wrappers.<OrderConsign>lambdaQuery().eq(OrderConsign::getOutTradeNo, outTradeNo));
		if(query == null ){
			logger.error("订单异常", "无订单数据！订单号：" + outTradeNo);
			return false;
		}
		Consign queryConsign = consignMapper.selectById(query.getConsignId());
		if(queryConsign == null){ //后台的数据为0或者寄售的
			logger.error("订单异常", "寄售无数据！订单号：" + outTradeNo);
			return false;
		}
		//更新订单
		Order order = new Order();
		//更新寄售状态为已售出
		if(queryConsign.getNowStock() == 0){
			Consign updateConsign = new Consign();
			updateConsign.setId(query.getId());
			updateConsign.setConsignState(3);//已售出
			int k =	consignMapper.updateById(updateConsign);
			if(k != 1){
				logger.error("订单异常", "更新寄售状态！订单号：" + outTradeNo);
				throw new ServiceException("更新寄售状态失败");
			}
		}
		//选择寄售方式为寄售则生成新的寄售
		if(query.getConsignType() == 2){
			Consign consign = new Consign();
			consign.setOrderId(queryOrder.getId()); //保存来源的订单ID
			consign.setConsignFor(2);//寄售开源订单
			consign.setConsignConfigId(queryConsign.getConsignConfigId());
			consign.setConsignDays(query.getConsignDays());
			consign.setConsignUserId(query.getPayUserId());//支付人变为寄售人
			consign.setConsignType(0);//无
			consign.setTotalNum(1);
			consign.setNowStock(1);
			consign.setIsDelivery(1); //是否交割 否
			consign.setIsSale(2); //是否上架 是（自动上架）
			consign.setConsignPrice(query.getConsignPrice());
			consign.setDeliveryPrice(0d);//默认为0，需要后台设置交割价
			consign.setGoodsId(query.getGoodsId());
			consign.setGoodsName(query.getGoodsName());
			consign.setTitle(query.getGoodsName());
			Double ratio = 0d;
			if(query.getConsignDays() == 2){
				ratio = Double.valueOf(ParamCache.getValue(CommonConstant.DEFAULT_CONDIGN_RATIO_T2));
			}else if(query.getConsignDays() == 3){
				ratio = Double.valueOf(ParamCache.getValue(CommonConstant.DEFAULT_CONDIGN_RATIO_T3));
			}else {
				ratio = Double.valueOf(ParamCache.getValue(CommonConstant.DEFAULT_CONDIGN_RATIO_T1));
			}
			Double ratioPrice = NumberUtil.mul(query.getConsignPrice(), ratio);
			Double obtainPrice = NumberUtil.add(query.getConsignPrice(), ratioPrice);
			consign.setObtainPrice(obtainPrice); //取得价
			//当前时间 + 寄售天数
			Date date = cn.hutool.core.date.DateUtil.offsetDay(DateUtil.now(), query.getConsignDays());
			consign.setStartTime(date);
			consign.setEndTime(DateUtil.plusMinutes(date,30));
			consign.setSaleDays(0);
			int j = consignMapper.insert(consign);
			if(j != 1){
				logger.error("订单异常", "订单变为寄售失败！订单号：" + outTradeNo);
				throw new ServiceException("订单变为寄售失败");
			}
		}
		order.setPayFlag(2);//待发货
		order.setPayTime(DateUtil.now()); //支付时间
		order.setPayCode(payCode); //支付方式
		int i = baseMapper.update(order,Wrappers.<Order>lambdaQuery().eq(Order::getOutTradeNo, outTradeNo));
		if (i != 1){
			logger.error("订单异常", "订单回调失败！订单号：" + outTradeNo);
			throw new ServiceException("订单回调失败！");
		}
		//更新用户数据
		UserApp userApp = new UserApp();
		UserApp queryUserApp = userApp.selectOne(Wrappers.<UserApp>lambdaQuery().eq(UserApp::getUserId,queryOrder.getUserId()));
		if (queryUserApp == null){
			logger.error("订单异常", "获取用户信息失败！订单号：" + outTradeNo);
			throw new ServiceException("获取用户信息失败！");
		}
		userApp.setId(queryUserApp.getId());
		Double ticket = NumberUtil.add(queryUserApp.getTicket(), queryOrder.getTotalTicket());
		userApp.setTicket(ticket);
		boolean flg = userApp.updateById();
		if (!flg){
			logger.error("订单异常", "更新用户信息失败！订单号：" + outTradeNo);
			throw new ServiceException("更新用户信息失败！");
		}
		return true;
	}

	@Override
	public boolean orderEnd(Long orderId, Long userId) {
		Order order = baseMapper.selectById(orderId);
		//必须是支付状态为待收货
		if (order.getPayFlag() != 3){
			throw new ServiceException("请等待商家发货!");
		}
		Order param = new Order();
		param.setId(orderId);
		param.setPayFlag(4); //已完成
		param.setRecordTime(DateUtil.now());
		int i = baseMapper.updateById(param);
		return i == 1;
	}

}
